#include "xparameters.h"
#include "xbasic_types.h"   /* Xilinx basic types for device drivers        */
#include "xac97.h"
#include "xac97_l.h"
#include "xac97_i.h"
#include "mb_interface.h"
#include "xuartlite_l.h"
#include "sysace.h"

#define TRUE 1
#define FALSE 0
#define printf xil_printf

#define MEM_BASE_ADDR 0x80300000
#define MEM_TOP_ADDR  0x803FFFFF

Xuint16* sound = (Xuint16*)MEM_BASE_ADDR;

void noEffect()
{
	Xuint16 soundbyte;
	printf("no effect\n\r");
	while( sound < (Xuint16*)MEM_TOP_ADDR &&
	       XUartLite_mIsReceiveEmpty(XPAR_RS232_BASEADDR) )
	{
		soundbyte = *sound;
		microblaze_bwrite_datafsl(soundbyte, 0);
		sound++;
	}
}

void slowDown()
{
	Xuint16 soundbyte;
	printf("low freq\n\r");
	while( sound < (Xuint16*)MEM_TOP_ADDR &&
	       XUartLite_mIsReceiveEmpty(XPAR_RS232_BASEADDR) )
	{
		soundbyte = *sound;
		microblaze_bwrite_datafsl(soundbyte, 0);
		microblaze_bwrite_datafsl(soundbyte, 0);
		sound++;
	}
}

void fastForward()
{
	Xuint16 soundbyte;
	printf("high freq\n\r");
	while( sound < (Xuint16*)MEM_TOP_ADDR &&
	       XUartLite_mIsReceiveEmpty(XPAR_RS232_BASEADDR) )
	{
		soundbyte = *sound;
		microblaze_bwrite_datafsl(soundbyte, 0);
		sound+=2;
	}
}

void echo()
{
	Xuint16 soundbyte;
	printf("echo\n\r");
	while( sound < (Xuint16*)MEM_TOP_ADDR &&
	       XUartLite_mIsReceiveEmpty(XPAR_RS232_BASEADDR) )
	{
		soundbyte = *sound;
		microblaze_bwrite_datafsl(soundbyte,1);
		microblaze_bread_datafsl(soundbyte,1);
		microblaze_bwrite_datafsl(soundbyte, 0);
		sound++;
	}
}

void record()
{
	Xuint16 soundbyte;

	sound = (Xuint16*)MEM_BASE_ADDR;
	printf("Recording\n\r" );
	while( sound < (Xuint16*)MEM_TOP_ADDR && 
	       XUartLite_mIsReceiveEmpty(XPAR_RS232_BASEADDR) )
	{
		microblaze_bread_datafsl(soundbyte, 0);
		*sound = soundbyte;
		sound++;
	}
}

void playback()
{
	Xuint16 soundbyte;
	Xuint8 keyPressed;
	printf("Playback....\r\n" );
	sound = (Xuint16*)MEM_BASE_ADDR;

	noEffect();

	while( sound < (Xuint16*)MEM_TOP_ADDR )
	{
		keyPressed = XUartLite_RecvByte(XPAR_RS232_BASEADDR);
		if( keyPressed == '1' )
			noEffect();
		else if( keyPressed == '2' )
			slowDown();
		else if( keyPressed == '3' )
			fastForward();
		else if( keyPressed = '4' )
			echo();
	}
	printf("Playback Complete.\n\n\r");
}

void saveCF()
{
	int i;
	printf("Saving to CF\n\r");
	sound = (Xuint16*)MEM_BASE_ADDR;
	for( i = 0; i < 1300; i++ )
	{
		write_sector( i+1, (BYTE*)sound );
		sound = sound + 256;
		if( i % 130 == 0 )
			printf("\r %d\%", i/13 );
	}
	printf("\r Done.\n\r");
}

void loadCF()
{
	int i;
	printf("Loading from CF...\n\r");
	sound = (Xuint16*)MEM_BASE_ADDR;
	for( i = 0; i < 1300; i++ )
	{
		read_sector( i+1, (BYTE*)sound );
		sound = sound + 256;
		if( i % 130 == 0 )
			printf("\r %d\%", i/13 );
	}
	printf("\r Done.\r\n\n");
}

void clearMem()
{
	printf("Clear memory...");
	sound = (Xuint16*)MEM_BASE_ADDR;
	memset( MEM_BASE_ADDR, 0, (MEM_TOP_ADDR-MEM_BASE_ADDR) );
	printf("DONE\r\n");
}

int main (void) 
{
 	XStatus Status;
 	Xuint16 soundbyte;
	Xuint8 keyPressed;
	int i;

	printf("Starting Application\n\r");

 	init_sound (22000);
 
	playback_enable();
	record_enable();
	init_ace();
	
 	while (1) 
	{

		printf("Press r to record. p to playback\n\r");
	
		keyPressed = XUartLite_RecvByte(XPAR_RS232_BASEADDR);
		printf( "%c\n\r", keyPressed );
		
		if( keyPressed == 'r' )
		{
			record();
		}
		else if( keyPressed =='p' )
		{
			playback();
		}
		else if( keyPressed == 's' )
		{
			saveCF();
		}
		else if( keyPressed == 'l' )
		{
			loadCF();
		}
		else if( keyPressed == 'c' )
		{
			clearMem();
		}
	}
}