LIBRARY ieee;
USE ieee.std_logic_1164.ALL;

ENTITY fir_filter IS
	generic (
		C_DWIDTH	: integer := 32
	);
	
	port (
	-- Global signals
	Clk		: IN std_logic;
	Reset		: IN std_logic;
	
	-- Master FSL signals
	FSL_M_DATA	: OUT std_logic_vector(0 to C_DWIDTH-1);	 -- output from the processor 16 bits
	FSL_M_CONTROL	: OUT std_logic;
	FSL_M_WRITE	: OUT std_logic;
	FSL_M_FULL	: IN std_logic;
	FSL_M_CLK	: OUT std_logic;
	
	-- Slave FSL signals
	FSL_S_DATA	: IN std_logic_vector(0 to C_DWIDTH-1);  -- output from the core is 32 bits
	FSL_S_CONTROL	: IN std_logic;
	FSL_S_READ	: OUT std_logic;
	FSL_S_EXISTS	: IN std_logic;
	FSL_S_CLK	: OUT std_logic
	);
END fir_filter;

ARCHITECTURE behavioral OF fir_filter IS

	signal temp		:std_logic_vector(15 downto 0);
     	signal temp2		:std_logic_vector(35 downto 0);
	signal temp3		:std_logic;

component my_filter
	port (
	ND: IN std_logic;
	RDY: OUT std_logic;
	CLK: IN std_logic;
	RST: IN std_logic;
	RFD: OUT std_logic;
	DIN: IN std_logic_VECTOR(15 downto 0);
	DOUT: OUT std_logic_VECTOR(35 downto 0));
end component;

BEGIN

FSL_S_READ <= '1'; -- always get data from FSL
temp <= FSL_S_DATA(0 to 15); 	-- data coming from FSL is 32 bits, we
					--only need the first 16 bits

my_filter_instance : my_filter
		port map (
			ND => FSL_S_EXISTS,
			RDY => FSL_M_WRITE,
			CLK => Clk,
			RST => Reset,
			RFD => FSL_S_READ, -- this is not really needed since we are constantly reading from the FSL
			DIN => FSL_S_DATA(0 to 15),
			DOUT(31 downto 0) => FSL_M_DATA);

--FSL_M_DATA <= temp2(31 downto 0);	 -- chop out unwanted 4 bits

END behavioral;
