-------------------------------------------------------------------------------
-- $Id: fsl_v20.vhd,v 1.1 2003/10/02 21:56:22 abq_ip Exp $
-------------------------------------------------------------------------------
-- fsl_v20.vhd - Entity and architecture
--
--  ***************************************************************************
--  **  Copyright(C) 2003 by Xilinx, Inc. All rights reserved.               **
--  **                                                                       **
--  **  This text contains proprietary, confidential                         **
--  **  information of Xilinx, Inc. , is distributed by                      **
--  **  under license from Xilinx, Inc., and may be used,                    **
--  **  copied and/or disclosed only pursuant to the terms                   **
--  **  of a valid license agreement with Xilinx, Inc.                       **
--  **                                                                       **
--  **  Unmodified source code is guaranteed to place and route,             **
--  **  function and run at speed according to the datasheet                 **
--  **  specification. Source code is provided "as-is", with no              **
--  **  obligation on the part of Xilinx to provide support.                 **
--  **                                                                       **
--  **  Xilinx Hotline support of source code IP shall only include          **
--  **  standard level Xilinx Hotline support, and will only address         **
--  **  issues and questions related to the standard released Netlist        **
--  **  version of the core (and thus indirectly, the original core source). **
--  **                                                                       **
--  **  The Xilinx Support Hotline does not have access to source            **
--  **  code and therefore cannot answer specific questions related          **
--  **  to source HDL. The Xilinx Support Hotline will only be able          **
--  **  to confirm the problem in the Netlist version of the core.           **
--  **                                                                       **
--  **  This copyright and support notice must be retained as part           **
--  **  of this text at all times.                                           **
--  ***************************************************************************
--
-------------------------------------------------------------------------------
-- Filename:        fsl_v20.vhd
--
-- Description:     
--                  
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--              fsl_v20.vhd
--
-------------------------------------------------------------------------------
-- Author:          satish
-- Revision:        $Revision: 1.1 $
-- Date:            $Date: 2003/10/02 21:56:22 $
--
-- History:
--   satish  2003-02-13    First Version
--
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;

library Unisim;
use Unisim.all;

entity fsl_v20 is
  generic (
    C_EXT_RESET_HIGH  : integer := 1;
    C_FSL_DWIDTH      : integer := 32;    
    C_FSL_DEPTH       : integer := 16;
    C_USE_CONTROL     : integer := 1
    );
  port (
    -- Clock and reset signals
    FSL_Clk : in  std_logic;
    SYS_Rst : in  std_logic;
    FSL_Rst : out std_logic;

    -- FSL master signals
    FSL_M_Clk     : in  std_logic;
    FSL_M_Data    : in  std_logic_vector(0 to C_FSL_DWIDTH-1);
    FSL_M_Control : in  std_logic;
    FSL_M_Write   : in  std_logic;
    FSL_M_Full    : out std_logic;

    -- FSL slave signals
    FSL_S_Clk     : in  std_logic;
    FSL_S_Data    : out std_logic_vector(0 to C_FSL_DWIDTH-1);
    FSL_S_Control : out std_logic;
    FSL_S_Read    : in  std_logic;
    FSL_S_Exists  : out std_logic 
    );
end entity fsl_v20;

architecture IMP of fsl_v20 is

  component SRL16 is
-- synthesis translate_off
    generic (
      INIT : bit_vector);
-- synthesis translate_on
    port (D   : in  std_logic;
          CLK : in  std_logic;
          A0  : in  std_logic;
          A1  : in  std_logic;
          A2  : in  std_logic;
          A3  : in  std_logic;
          Q   : out std_logic); 
  end component SRL16;

  component FDS is
    port(
      Q : out std_logic;
      D : in  std_logic;
      C : in  std_logic;
      S : in  std_logic);
  end component FDS;

  component SRL_FIFO is
    generic (
      C_DATA_BITS : integer;
      C_DEPTH     : integer);
    port (
      Clk             : in  std_logic;
      Reset           : in  std_logic;
      FIFO_Write      : in  std_logic;
      Data_In         : in  std_logic_vector(0 to C_DATA_BITS-1);
      FIFO_Read       : in  std_logic;
      Data_Out        : out std_logic_vector(0 to C_DATA_BITS-1);
      FIFO_Full       : out std_logic;
      FIFO_Half_Full  : out std_logic;
      FIFO_Half_Empty : out std_logic;
      Data_Exists     : out std_logic);
  end component SRL_FIFO;

  component raminfr is 
    generic (
      C_DWIDTH : integer := 32;
      C_AWIDTH : integer := 16
      );
    port ( 
      clk  : in std_logic; 
      we   : in std_logic; 
      a    : in std_logic_vector(C_AWIDTH-1 downto 0); 
      dpra : in std_logic_vector(C_AWIDTH-1 downto 0); 
      di   : in std_logic_vector(C_DWIDTH-1 downto 0); 
      spo  : out std_logic_vector(C_DWIDTH-1 downto 0); 
      dpo  : out std_logic_vector(C_DWIDTH-1 downto 0) 
      ); 
  end component raminfr; 

  signal sys_rst_i    : std_logic;
  signal srl_time_out : std_logic;
  signal fsl_rst_i    : std_logic;
  signal Data_In      : std_logic_vector(0 to C_FSL_DWIDTH);
  signal Data_Out     : std_logic_vector(0 to C_FSL_DWIDTH);

  signal read_addr_ptr      : natural;
  signal write_addr_ptr     : natural;

  signal read_addr_ptr_i      : natural;
  signal write_addr_ptr_i     : natural;

  signal fsl_m_write_i    : std_logic;
  signal fsl_s_exists_i   : std_logic;
  signal fsl_m_full_i     : std_logic;
  
  signal fifo_length      : natural range 0 to C_FSL_DEPTH-1;
  attribute INIT              : string;
  attribute INIT of POR_SRL_I : label is "FFFF";

  -- purpose: calculate width of used address bits for address range  occupied by the
  -- memory. Requires 2^N sized blocks
  function addr_used_width (
    addr_size : integer)
    return integer is
    variable addr_cnt : integer := 0;
    variable cnt : integer := 0;
  begin  -- addr_decode_width
    addr_cnt := addr_size;
    while addr_cnt > 0 loop
      addr_cnt := addr_cnt / 2;
      cnt := cnt + 1;
    end loop;
    return cnt-1;
  end addr_used_width;

  constant AddrWidth : integer := addr_used_width(C_FSL_DEPTH);
  signal Read_Address       : std_logic_vector(0 to AddrWidth-1);
  signal Write_Address      : std_logic_vector(0 to AddrWidth-1);


begin  -- architecture IMP

  SYS_RST_PROC : process (SYS_Rst) is
    variable sys_rst_input : std_logic;
  begin
    if C_EXT_RESET_HIGH = 0 then
      sys_rst_i <= not SYS_Rst;
    else
      sys_rst_i <= SYS_Rst;
    end if;
  end process SYS_RST_PROC;
  
  POR_SRL_I : SRL16
-- synthesis translate_off
    generic map (
      INIT => X"FFFF") 
-- synthesis translate_on
    port map (
      D   => '0',
      CLK => FSL_Clk,
      A0  => '1',
      A1  => '1',
      A2  => '1',
      A3  => '1',
      Q   => srl_time_out);
  
  POR_FF_I : FDS
    port map (
      Q => fsl_rst_i,
      D => srl_time_out,
      C => FSL_Clk,
      S => sys_rst_i);
      
  FSL_Rst <= fsl_rst_i;

  
  srl_fifo_imp: if (C_FSL_DEPTH = 16) generate      

    data_and_control_srl: if (C_USE_CONTROL = 1) generate
      Data_In(0 to C_FSL_DWIDTH-1) <= FSL_M_Data;
      Data_In(C_FSL_DWIDTH)        <= FSL_M_Control;

      -- map FIFO to Master/Slave signals
      FSL_FIFO : SRL_FIFO
	generic map (
          C_DATA_BITS => C_FSL_DWIDTH + 1,
	  C_DEPTH     => C_FSL_DEPTH)
	port map (
          Clk             => FSL_Clk,
	  Reset           => fsl_rst_i,
	  FIFO_Write      => FSL_M_Write,    -- Master Write Signal
	  Data_In         => Data_In,        -- Master Data
	  FIFO_Read       => FSL_S_Read,     -- Slave Read Signal
	  Data_Out        => Data_Out,       -- Slave Data
	  FIFO_Full       => FSL_M_Full,     -- FIFO full signal
	  FIFO_Half_Full  => open,
	  FIFO_Half_Empty => open,
	  Data_Exists     => FSL_S_Exists);  -- Slave Data exists
      
      -- Map Slave Data/Control signal
      FSL_S_Data    <= Data_Out(0 to C_FSL_DWIDTH-1);
      FSL_S_Control <= Data_Out(C_FSL_DWIDTH);
        
    end generate data_and_control_srl;
    
    data_only_srl: if (C_USE_CONTROL = 0) generate
      -- map FIFO to Master/Slave signals
      FSL_FIFO : SRL_FIFO
	generic map (
          C_DATA_BITS => C_FSL_DWIDTH,
	  C_DEPTH     => C_FSL_DEPTH)
	port map (
          Clk             => FSL_Clk,
	  Reset           => fsl_rst_i,
	  FIFO_Write      => FSL_M_Write,    -- Master Write Signal
	  Data_In         => FSL_M_Data,        -- Master Data
	  FIFO_Read       => FSL_S_Read,     -- Slave Read Signal
	  Data_Out        => FSL_S_Data,       -- Slave Data
	  FIFO_Full       => FSL_M_Full,     -- FIFO full signal
	  FIFO_Half_Full  => open,
	  FIFO_Half_Empty => open,
	  Data_Exists     => FSL_S_Exists);  -- Slave Data exists
      
    end generate data_only_srl;
        
  end generate srl_fifo_imp;

  bram_imp: if (C_FSL_DEPTH > 16) generate

    -- Increment Read Address Pointer
    Read_Addr_Handle : process (FSL_Clk, fsl_rst_i)
    begin
      if (fsl_rst_i = '1') then
	read_addr_ptr <= 0;
      elsif (FSL_Clk'event and FSL_Clk = '1') then
	if (FSL_S_Read = '1') then
	  read_addr_ptr <= (read_addr_ptr + 1) mod (2 ** AddrWidth);
	end if;
      end if;
    end process Read_Addr_Handle;

    -- Increment Write Address Pointer
    Write_Addr_Handle : process (FSL_Clk, fsl_rst_i)
    begin
      if (fsl_rst_i = '1') then
	write_addr_ptr <= 0;
      elsif (FSL_Clk'event and FSL_Clk = '1') then
	if (FSL_M_Write = '1') then
	  write_addr_ptr <= (write_addr_ptr + 1) mod (2 ** AddrWidth);
	end if;
      end if;
    end process Write_Addr_Handle;

    -- FIFO length handling
    Fifo_Length_Handle : process (FSL_Clk, fsl_rst_i)
    begin
      if (fsl_rst_i = '1') then
	fifo_length <= 0;
      elsif (FSL_Clk'event and FSL_Clk = '1') then
	-- write and no read => increment length
	-- don't increment length when FULL
	if (FSL_M_Write = '1' and FSL_S_Read = '0' and fsl_m_full_i = '0') then
	  fifo_length <= fifo_length + 1;
        -- read and no write => decrement length
	-- don't decrement length when EMPTY
	elsif (FSL_M_Write = '0' and FSL_S_Read = '1' and fsl_s_exists_i = '1') then
	  fifo_length <= fifo_length - 1;
	end if;
      end if;
    end process Fifo_Length_Handle;

    Write_Address <= std_logic_vector(to_unsigned(write_addr_ptr, AddrWidth));
    Read_Address  <= std_logic_vector(to_unsigned(read_addr_ptr, AddrWidth));

    -- Set Full and empty flags
    fsl_m_full_i   <= '1' when (fifo_length = C_FSL_DEPTH-1) else '0';
    fsl_s_exists_i <= '1' when (fifo_length /= 0) else '0';

    FSL_M_Full   <= fsl_m_full_i;
    FSL_S_Exists <= fsl_s_exists_i;


    data_and_control_bram: if (C_USE_CONTROL = 1) generate
      Data_In(0 to C_FSL_DWIDTH-1) <= FSL_M_Data;
      Data_In(C_FSL_DWIDTH)        <= FSL_M_Control;

      BRAM_INST : raminfr
	generic map (
          C_DWIDTH => C_FSL_DWIDTH + 1,
	  C_AWIDTH => AddrWidth
	  )
	port map  (
          clk   => FSL_Clk,
	  we    => fsl_M_Write,
	  a     => Write_Address,
	  dpra  => Read_Address,
	  di    => Data_In,
	  spo   => open,
	  dpo   => Data_Out);

      -- Map Slave Data/Control signal
      FSL_S_Data    <= Data_Out(0 to C_FSL_DWIDTH-1);
      FSL_S_Control <= Data_Out(C_FSL_DWIDTH);
      
    end generate data_and_control_bram;
    
    data_only_bram: if (C_USE_CONTROL = 0) generate
      BRAM_INST : raminfr
	generic map (
          C_DWIDTH => C_FSL_DWIDTH,
	  C_AWIDTH => AddrWidth
	  )
	port map  (
          clk   => FSL_Clk,
	  we    => FSL_M_Write,
	  a     => Write_Address,
	  dpra  => Read_Address,
	  di    => FSL_M_Data,
	  spo   => open,
	  dpo   => FSL_S_Data);

    end generate data_only_bram;
    
  end generate bram_imp;

    
end architecture IMP;
