LIBRARY ieee;
USE ieee.std_logic_1164.ALL;

library fsl_v20_v2_00_a;
use fsl_v20_v2_00_a.fsl_v20;

ENTITY toplevel IS
  generic (
    C_EXT_RESET_HIGH : integer := 1;
    C_ASYNC_CLKS     : integer := 0;
    C_IMPL_STYLE     : integer := 0;
    C_USE_CONTROL    : integer := 1;
    C_FSL_DWIDTH     : integer := 32;
    C_FSL_DEPTH      : integer := 32
    );
	port (
	FSL_M_DATA	: IN std_logic_vector(C_FSL_DWIDTH-1 downto 0);	 -- output from the processor 16 bits
	FSL_M_CONTROL	: IN std_logic;
	FSL_M_WRITE	: IN std_logic;
	FSL_M_FULL	: OUT std_logic;
	FSL_S_DATA	: OUT std_logic_vector(C_FSL_DWIDTH-1 downto 0);  -- output from the core is 32 bits
	FSL_S_CONTROL	: OUT std_logic;
	FSL_S_READ	: IN std_logic;
	FSL_S_EXISTS	: OUT std_logic;
	SYS_CLK		: IN std_logic;
	SYS_RESET		: IN std_logic);
END toplevel;

ARCHITECTURE behavioral OF toplevel IS

	signal fsl_0_S_Data		:std_logic_vector(C_FSL_DWIDTH-1 downto 0); -- input from microblaze
	signal fsl_0_S_Control	:std_logic;
	signal fsl_0_S_Read		:std_logic;
	signal fsl_0_S_Exists	:std_logic;

	signal fsl_1_M_Data		:std_logic_vector(C_FSL_DWIDTH-1 downto 0);	-- output back to microblaze
	signal fsl_1_M_Control	:std_logic;
	signal fsl_1_M_Write		:std_logic;
	signal fsl_1_M_Full		:std_logic;

	signal temp					:std_logic_vector(35 downto 0);
   signal temp2				:std_logic_vector(31 downto 0);
	signal temp3				:std_logic_vector(31 downto 0);


component my_fsl
	Port (
	 	-- Global signals
      CLK : in std_logic;           -- System clock
      RESET : in std_logic;             -- System reset

		-- Slave FSL Signals
		FSL_S_Clk     : out std_logic;
		FSL_S_Read    : out std_logic;
		FSL_S_Data    : in  std_logic_vector(0 to C_FSL_DWIDTH-1);
		FSL_S_Control : in  std_logic;
		FSL_S_Exists  : in  std_logic;

		-- Master FSL Signals
		FSL_M_Clk     : out std_logic;
		FSL_M_Write   : out std_logic;
		FSL_M_Data    : out std_logic_vector(0 to C_FSL_DWIDTH-1);
		FSL_M_Control : out std_logic;
		FSL_M_Full    : in  std_logic
      );
end component;

component fsl_v20
  port (
    -- Clock and reset signals
    FSL_Clk : in  std_logic;
    SYS_Rst : in  std_logic;
    FSL_Rst : out std_logic;

    -- FSL master signals
    FSL_M_Clk     : in  std_logic;
    FSL_M_Data    : in  std_logic_vector(0 to C_FSL_DWIDTH-1);
    FSL_M_Control : in  std_logic;
    FSL_M_Write   : in  std_logic;
    FSL_M_Full    : out std_logic;

    -- FSL slave signals
    FSL_S_Clk     : in  std_logic;
    FSL_S_Data    : out std_logic_vector(0 to C_FSL_DWIDTH-1);
    FSL_S_Control : out std_logic;
    FSL_S_Read    : in  std_logic;
    FSL_S_Exists  : out std_logic;

    -- FIFO status signals
    FSL_Full     : out std_logic;
    FSL_Has_Data : out std_logic
    );
    
end component;

BEGIN

--temp2 <= "0000000000000000" & FSL_M_DATA; -- fill up the remaining 16 bits

fsl_0 : fsl_v20
		port map (
			FSL_CLK => SYS_CLK,
			FSL_S_CLK => SYS_CLK, 
			FSL_M_CLK => SYS_CLK,
			SYS_Rst => '0',
			FSL_M_Data => FSL_M_DATA, 
			FSL_M_Control => FSL_M_CONTROL,
			FSL_M_Write => FSL_M_WRITE,
			FSL_M_Full => FSL_M_FULL,
			FSL_S_Exists => fsl_0_S_Exists,
			FSL_S_Control => fsl_0_S_Control,
			FSL_S_Read => fsl_0_S_Read,
			FSL_S_Data => fsl_0_S_Data);

--fsl_0_S_Data <= temp3(15 downto 0);

fsl_1 : fsl_v20
		port map (
			FSL_CLK => SYS_CLK,
			FSL_S_CLK => SYS_CLK,
			FSL_M_CLK => SYS_CLK,
			SYS_Rst => '0',
			FSL_S_Data => FSL_S_DATA,
			FSL_S_Control => FSL_S_CONTROL,
			FSL_S_Read => FSL_S_READ,
			FSL_S_Exists => FSL_S_EXISTS,
			FSL_M_Data => fsl_1_M_Data,
			FSL_M_Control => fsl_1_M_Control,
			FSL_M_Write => fsl_1_M_Write,
			FSL_M_Full => fsl_1_M_Full);


my_filter_instance : my_fsl
		port map (
		   -- Global signals
         CLK => SYS_CLK,
         RESET => '0',            -- System reset
		   FSL_S_Read => fsl_0_S_Read,
		   FSL_S_Data => fsl_0_S_Data,
		   FSL_S_Control => fsl_0_S_Control,
		   FSL_S_Exists  => fsl_0_S_Exists,
		   FSL_M_Write => fsl_1_M_Write,
		   FSL_M_Data => fsl_1_M_Data,
		   FSL_M_Control => fsl_1_M_Control,
		   FSL_M_Full => fsl_1_M_Full);
			
--fsl_1_M_DATA <= temp(31 downto 0);	 -- chop out unwanted 4 bits


--process (SYS_CLK, SYS_RESET)
	--if (clk'EVENT and SYS_CLK = '1') then
			--if ( fsl_1_M_Control = '0') then
				--fsl_0_S_Read = '0';
			--end if;
	--end if;
--end process;

END behavioral;
